@@ -89,7 +89,7 @@ namespace QInstaller {
 */
 
 /*!
-    \enum LoggingHandler::VerbosityLevel
+    \enum QInstaller::LoggingHandler::VerbosityLevel
     \brief This enum holds the possible levels of output verbosity.
 
     \value Silent
@@ -384,7 +384,7 @@ void LoggingHandler::printAliasInformation(const QList<ComponentAlias *> &aliase
     QTextStream stream(&output);
 
     stream << Qt::endl;
-    for (auto *alias : qAsConst(sortedAliases)) {
+    for (auto *alias : std::as_const(sortedAliases)) {
         stream << "Name: " << alias->name() << Qt::endl;
         stream << "Display name: " << alias->displayName() << Qt::endl;
         stream << "Description: " << alias->description() << Qt::endl;
